Player = Class{}
require 'src/Animation'

local MOVE_SPEED = 100
local jumpVelocity = 400
local Gravity = 20

function Player:init(map)
    self.width = 17
    self.height = 32

    self.x = 10
    self.y = 130
    self.dx = 0
    self.dy = 0
    self.xOffset = 8
    self.yOffset = 16

    self.texture = love.graphics.newImage('graphics/mario.png')
    self.frames = generateQuads(self.texture, 17, 32)
    self.state = 'idle'
    self.direction = 'right'

    self.animations = {
        ['idle'] = Animation {
            texture = self.texture,
            frames = {
                self.frames[1]
            },
            interval = 1
        },
        ['walking'] = Animation {
            texture = self.texture,
            frames = {
                self.frames[2], self.frames[3], self.frames[4]
            },
            interval = 0.15
        },
        ['jumping'] = Animation {
            texture = self.texture,
            frames = {
                self.frames[11]
            },
            interval = 1
        }
    }
    self.animation = self.animations['idle']
    self.behaviors = {
        ['idle'] = function(dt)
            if love.keyboard.wasPressed('space') then
                self.dy = -jumpVelocity
                self.state = 'jumping'
                self.animation = self.animations['jumping']
            elseif love.keyboard.isDown('left') then
                self.dx = -MOVE_SPEED
                self.animation = self.animations['walking']
                self.direction = 'left'
            elseif love.keyboard.isDown('right') then
                self.dx = MOVE_SPEED
                self.animation = self.animations['walking']
                self.direction = 'right'
            else
                self.dx = 0
                self.animation = self.animations['idle']
            end
        end,
        ['walking'] = function(dt)
            if love.keyboard.wasPressed('space') then
                self.dy = -jumpVelocity
                self.state = 'jumping'
                self.animation = self.animations['jumping']
            elseif love.keyboard.isDown('left') then
                self.dx = -MOVE_SPEED
                self.animation = self.animations['walking']
                self.direction = 'left'

            elseif love.keyboard.isDown('right') then
                self.dx = MOVE_SPEED
                self.animation = self.animations['walking']
                self.direction = 'right'
            else
                self.dx = 0
                self.animation = self.animations['idle']
                
            end
        end,
        ['jumping'] = function(dt)
            if love.keyboard.isDown('left') then
                self.direction = 'left'
                self.dx = -MOVE_SPEED
            elseif love.keyboard.isDown('right') then
                self.direction = 'right'
                self.dx = MOVE_SPEED
            end
            self.dy = self.dy + Gravity

            if self.y >= 130 then
                self.y = 130
                self.dy = 0
                self.state = 'idle'
                self.animation = self.animations[self.state]
            end
        end
    }

end

function Player:update(dt)
    self.behaviors[self.state](dt)
    self.animation:update(dt)
    self.x = self.x + self.dx * dt
    self.y = self.y + self.dy * dt

end

function Player:render()
    local scaleX
    if self.direction == 'right' then
        scaleX = 1
    else
        scaleX = -1
    end

    love.graphics.draw(self.texture,self.animation:getCurrentFrame(),
        math.floor(self.x + self.xOffset), math.floor(self.y + self.yOffset),
        0, scaleX, 1,
        self.xOffset, self.yOffset)
end